@@ -95,6 +95,7 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " K e y " ]   =   " K e y " ; 
  l o c a l i z e d S t r i n g s [ " L O C A L   S T O R A G E " ]   =   " L O C A L   S T O R A G E " ; 
  l o c a l i z e d S t r i n g s [ " L o c a l " ]   =   " L o c a l " ; 
+ l o c a l i z e d S t r i n g s [ " L o c a t i o n " ]   =   " L o c a t i o n " ; 
  l o c a l i z e d S t r i n g s [ " M I M E   t y p e " ]   =   " M I M E   t y p e " ; 
  l o c a l i z e d S t r i n g s [ " M e t r i c s " ]   =   " M e t r i c s " ; 
  l o c a l i z e d S t r i n g s [ " N a m e " ]   =   " N a m e " ; 
@@ -127,12 +128,14 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " R e s o u r c e   i n t e r p r e t e d   a s   % s   b u t   t r a n s f e r r e d   w i t h   M I M E   t y p e   % s . " ]   =   " R e s o u r c e   i n t e r p r e t e d   a s   % 1 $ s   b u t   t r a n s f e r r e d   w i t h   M I M E   t y p e   % 2 $ s . " ; 
  l o c a l i z e d S t r i n g s [ " R e s o u r c e   t r a c k i n g   d i s a b l e d .   C l i c k   t o   e n a b l e . " ]   =   " R e s o u r c e   t r a c k i n g   d i s a b l e d .   C l i c k   t o   e n a b l e . " ; 
  l o c a l i z e d S t r i n g s [ " R e s o u r c e   t r a c k i n g   e n a b l e d .   C l i c k   t o   d i s a b l e . " ]   =   " R e s o u r c e   t r a c k i n g   e n a b l e d .   C l i c k   t o   d i s a b l e . " ; 
+ l o c a l i z e d S t r i n g s [ " R e s o u r c e " ]   =   " R e s o u r c e " ; 
  l o c a l i z e d S t r i n g s [ " R e s o u r c e s " ]   =   " R e s o u r c e s " ; 
  l o c a l i z e d S t r i n g s [ " R e s p o n s e   H e a d e r s " ]   =   " R e s p o n s e   H e a d e r s " ; 
  l o c a l i z e d S t r i n g s [ " R e s t o r e   a l l   f u n c t i o n s . " ]   =   " R e s t o r e   a l l   f u n c t i o n s . " ; 
  l o c a l i z e d S t r i n g s [ " R u n   % d " ]   =   " R u n   % d " ; 
  l o c a l i z e d S t r i n g s [ " S E S S I O N   S T O R A G E " ]   =   " S E S S I O N   S T O R A G E " ; 
  l o c a l i z e d S t r i n g s [ " S c o p e   V a r i a b l e s " ]   =   " S c o p e   V a r i a b l e s " ; 
+ l o c a l i z e d S t r i n g s [ " S c r i p t " ]   =   " S c r i p t " ; 
  l o c a l i z e d S t r i n g s [ " S c r i p t s " ]   =   " S c r i p t s " ; 
  l o c a l i z e d S t r i n g s [ " S e a r c h   % s " ]   =   " S e a r c h   % s " ; 
  l o c a l i z e d S t r i n g s [ " S e c u r e " ]   =   " S e c u r e " ; 
@@ -152,6 +155,7 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " S o r t   b y   L a t e n c y " ]   =   " S o r t   b y   L a t e n c y " ; 
  l o c a l i z e d S t r i n g s [ " S o r t   b y   R e s p o n s e   T i m e " ]   =   " S o r t   b y   R e s p o n s e   T i m e " ; 
  l o c a l i z e d S t r i n g s [ " S o r t   b y   S i z e " ]   =   " S o r t   b y   S i z e " ; 
+ l o c a l i z e d S t r i n g s [ " S o r t   b y   T r a n s f e r   S i z e " ]   =   " S o r t   b y   T r a n s f e r   S i z e " ; 
  l o c a l i z e d S t r i n g s [ " S o r t   b y   S t a r t   T i m e " ]   =   " S o r t   b y   S t a r t   T i m e " ; 
  l o c a l i z e d S t r i n g s [ " S o u r c e " ]   =   " S o u r c e " ; 
  l o c a l i z e d S t r i n g s [ " S t a r t   p r o f i l i n g . " ]   =   " S t a r t   p r o f i l i n g . " ; 
@@ -256,7 +260,6 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " S e l e c t   a u d i t s   t o   r u n " ]   =   " S e l e c t   a u d i t s   t o   r u n " ; 
  l o c a l i z e d S t r i n g s [ " S e l e c t   A l l " ]   =   " S e l e c t   A l l " ; 
  l o c a l i z e d S t r i n g s [ " R u n " ]   =   " R u n " ; 
- l o c a l i z e d S t r i n g s [ " R u n n i n g . . . " ]   =   " R u n n i n g . . . " ; 
  l o c a l i z e d S t r i n g s [ " A u d i t   P r e s e n t   S t a t e " ]   =   " A u d i t   P r e s e n t   S t a t e " ; 
  l o c a l i z e d S t r i n g s [ " A u d i t   P r e s e n t   S t a t e   ( R e s o u r c e   T r a c k i n g   m u s t   b e   e n a b l e d ) " ]   =   " A u d i t   P r e s e n t   S t a t e   ( R e s o u r c e   T r a c k i n g   m u s t   b e   e n a b l e d ) " ; 
  l o c a l i z e d S t r i n g s [ " N o   a u d i t s   t o   r u n " ]   =   " N o   a u d i t s   t o   r u n " ; 
@@ -272,5 +275,22 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " C o n t e n t " ]   =   " C o n t e n t " ; 
  l o c a l i z e d S t r i n g s [ " % d   c o o k i e s   ( % s ) " ]   =   " % d   c o o k i e s   ( % s ) " ; 
  l o c a l i z e d S t r i n g s [ " S h o w   A l l   N o d e s   ( % d   M o r e ) " ]   =   " S h o w   A l l   N o d e s   ( % d   M o r e ) " ; 
+ l o c a l i z e d S t r i n g s [ " F u n c t i o n   C a l l " ]   =   " F u n c t i o n   C a l l " ; 
+ l o c a l i z e d S t r i n g s [ " D u r a t i o n " ]   =   " D u r a t i o n " ; 
+ l o c a l i z e d S t r i n g s [ " T y p e " ]   =   " T y p e " ; 
+ l o c a l i z e d S t r i n g s [ " L o c a t i o n " ]   =   " L o c a t i o n " ; 
+ l o c a l i z e d S t r i n g s [ " D e t a i l s " ]   =   " D e t a i l s " ; 
+ l o c a l i z e d S t r i n g s [ " C a l l e r " ]   =   " C a l l e r " ; 
+ l o c a l i z e d S t r i n g s [ " T i m e o u t " ]   =   " T i m e o u t " ; 
+ l o c a l i z e d S t r i n g s [ " R e p e a t s " ]   =   " R e p e a t s " ; 
+ l o c a l i z e d S t r i n g s [ " C o d e   S i t e " ]   =   " C o d e   S i t e " ; 
+ l o c a l i z e d S t r i n g s [ " % s   -   D e t a i l s " ]   =   " % s   -   D e t a i l s " ; 
+ l o c a l i z e d S t r i n g s [ " T i m e r   I d " ]   =   " T i m e r   I d " ; 
  l o c a l i z e d S t r i n g s [ " A c t i v a t e   a l l   b r e a k p o i n t s . " ]   =   " A c t i v a t e   a l l   b r e a k p o i n t s . " ; 
- l o c a l i z e d S t r i n g s [ " D e a c t i v a t e   a l l   b r e a k p o i n t s . " ]   =   " D e a c t i v a t e   a l l   b r e a k p o i n t s . " ; 
\ No newline at end of file
+ l o c a l i z e d S t r i n g s [ " D e a c t i v a t e   a l l   b r e a k p o i n t s . " ]   =   " D e a c t i v a t e   a l l   b r e a k p o i n t s . " ; 
+ l o c a l i z e d S t r i n g s [ " % s   ( % s   t r a n s f e r r e d ) " ]   =   " % 1 $ s   ( % 2 $ s   t r a n s f e r r e d ) " ; 
+ l o c a l i z e d S t r i n g s [ " C a n n o t   o p e n   t h i s   l i n k .   M a k e   s u r e   t h a t   r e s o u r c e   t r a c k i n g   i s   e n a b l e d   i n   t h e   R e s o u r c e s   p a n e l . " ]   =   " C a n n o t   o p e n   t h i s   l i n k .   M a k e   s u r e   t h a t   r e s o u r c e   t r a c k i n g   i s   e n a b l e d   i n   t h e   R e s o u r c e s   p a n e l . " ; 
+ l o c a l i z e d S t r i n g s [ " W o r k e r s " ]   =   " W o r k e r s " ; 
+ l o c a l i z e d S t r i n g s [ " D e b u g " ]   =   " D e b u g " ; 
+ l o c a l i z e d S t r i n g s [ " A l l o w   d e b u g g i n g   w o r k e r s .   E n a b l i n g   t h i s   o p t i o n   w i l l   r e p l a c e   n a t i v e   w o r k e r s   w i t h   t h e   i f r a m e - b a s e d   J a v a S c r i p t   i m p l e m e n t a t i o n " ]   =   " A l l o w   d e b u g g i n g   w o r k e r s .   E n a b l i n g   t h i s   o p t i o n   w i l l   r e p l a c e   n a t i v e   w o r k e r s   w i t h   t h e   i f r a m e - b a s e d   J a v a S c r i p t   i m p l e m e n t a t i o n " ; 
+ l o c a l i z e d S t r i n g s [ " L o a d i n g   ( % d   o f   % d ) " ]   =   " L o a d i n g   ( % d   o f   % d ) " ; 
\ No newline at end of file
