@@ -378,7 +378,7 @@ void ScriptDebugServer::setJavaScriptPaused(Frame* frame, bool paused)
 {
     ASSERT_ARG(frame, frame);
 
-    if (!frame->script()->canExecuteScripts())
+    if (!frame->script()->canExecuteScripts(NotAboutToExecuteScript))
         return;
 
     frame->script()->setPaused(paused);
@@ -392,14 +392,6 @@ void ScriptDebugServer::setJavaScriptPaused(Frame* frame, bool paused)
     setJavaScriptPaused(frame->view(), paused);
 }
 
-#if PLATFORM(MAC)
-
-void ScriptDebugServer::setJavaScriptPaused(FrameView*, bool)
-{
-}
-
-#else
-
 void ScriptDebugServer::setJavaScriptPaused(FrameView* view, bool paused)
 {
     if (!view)
@@ -417,8 +409,6 @@ void ScriptDebugServer::setJavaScriptPaused(FrameView* view, bool paused)
     }
 }
 
-#endif
-
 void ScriptDebugServer::pauseIfNeeded(Page* page)
 {
     if (m_paused)
