@@ -1,6 +1,4 @@
 /**
- * This file is part of the DOM implementation for KDE.
- *
  * Copyright (C) 2001 Peter Kelly (pmk@post.com)
  * Copyright (C) 2001 Tobias Anton (anton@stud.fbi.fh-darmstadt.de)
  * Copyright (C) 2006 Samuel Weinig (sam.weinig@gmail.com)
@@ -29,10 +27,8 @@
 
 namespace WebCore {
 
-using namespace EventNames;
-
 BeforeUnloadEvent::BeforeUnloadEvent()
-    : Event(beforeunloadEvent, false, true)
+    : Event(eventNames().beforeunloadEvent, false, true)
 {
 }
 
