@@ -30,14 +30,9 @@
 
 namespace WebCore {
     
-    class CachedFramePlatformData;
-    class DOMWindow;
-    class Document;
-    class DocumentLoader;
-    class FrameView;
-    class KURL;
-    class Node;
-    class Page;
+class Document;
+class DocumentLoader;
+class Page;
 
 class CachedPage : public RefCounted<CachedPage> {
 public:
@@ -46,12 +41,10 @@ public:
 
     void restore(Page*);
     void clear();
+    void destroy();
 
     Document* document() const { return m_cachedMainFrame->document(); }
     DocumentLoader* documentLoader() const { return m_cachedMainFrame->documentLoader(); }
-    FrameView* mainFrameView() const { return m_cachedMainFrame->view(); }
-    const KURL& url() const { return m_cachedMainFrame->url(); }
-    DOMWindow* domWindow() const { return m_cachedMainFrame->domWindow(); }
 
     double timeStamp() const { return m_timeStamp; }
     
