@@ -33,20 +33,28 @@ WebInspector.TimelineAgent = function() {
 }
 
 // Must be kept in sync with TimelineItem.h
-WebInspector.TimelineAgent.ItemType = {
-    DOMDispatch       : 0,
-    Layout            : 1,
+WebInspector.TimelineAgent.RecordType = {
+    EventDispatch : 0,
+    Layout : 1,
     RecalculateStyles : 2,
-    Paint             : 3,
-    ParseHTML         : 4,
-    TimerInstall      : 5,
-    TimerRemove       : 6,
-    TimerFire         : 7,    
+    Paint : 3,
+    ParseHTML : 4,
+    TimerInstall : 5,
+    TimerRemove : 6,
+    TimerFire : 7,
+    XHRReadyStateChange : 8,
+    XHRLoad : 9,
+    EvaluateScript : 10,
+    MarkTimeline : 11,
+    ResourceSendRequest : 12,
+    ResourceReceiveResponse : 13,
+    ResourceFinish : 14,
+    FunctionCall : 15
 };
 
-WebInspector.addItemToTimeline = function(record) {
+WebInspector.addRecordToTimeline = function(record) {
     if (WebInspector.panels.timeline)
-        WebInspector.panels.timeline.addItemToTimeline(record);
+        WebInspector.panels.timeline.addRecordToTimeline(record);
 }
 
 WebInspector.timelineProfilerWasStarted = function() {
