@@ -26,8 +26,6 @@
 #ifndef Cursor_h
 #define Cursor_h
 
-#include <wtf/Platform.h>
-
 #if PLATFORM(WIN)
 typedef struct HICON__* HICON;
 typedef HICON HCURSOR;
@@ -40,6 +38,8 @@ typedef struct _GdkCursor GdkCursor;
 #include <QCursor>
 #elif PLATFORM(CHROMIUM)
 #include "PlatformCursor.h"
+#elif PLATFORM(HAIKU)
+#include <app/Cursor.h>
 #endif
 
 #if PLATFORM(MAC)
@@ -54,6 +54,11 @@ class NSCursor;
 class wxCursor;
 #endif
 
+#if PLATFORM(WIN)
+typedef struct HICON__ *HICON;
+typedef HICON HCURSOR;
+#endif
+
 namespace WebCore {
 
     class Image;
@@ -63,7 +68,7 @@ namespace WebCore {
     class SharedCursor : public RefCounted<SharedCursor> {
     public:
         static PassRefPtr<SharedCursor> create(HCURSOR nativeCursor) { return adoptRef(new SharedCursor(nativeCursor)); }
-        ~SharedCursor() { DestroyIcon(m_nativeCursor); }
+        ~SharedCursor();
         HCURSOR nativeCursor() const { return m_nativeCursor; }
     private:
         SharedCursor(HCURSOR nativeCursor) : m_nativeCursor(nativeCursor) { }
@@ -86,6 +91,9 @@ namespace WebCore {
 #elif PLATFORM(CHROMIUM)
     // See PlatformCursor.h
     typedef void* PlatformCursorHandle;
+#elif PLATFORM(HAIKU)
+    typedef BCursor* PlatformCursor;
+    typedef BCursor* PlatformCursorHandle;
 #else
     typedef void* PlatformCursor;
     typedef void* PlatformCursorHandle;
