@@ -45,7 +45,6 @@
 #include "Pattern.h"
 #include "SimpleFontData.h"
 #include "SourceGraphic.h"
-#include "TransformationMatrix.h"
 
 #include <cairo.h>
 #include <math.h>
@@ -76,10 +75,12 @@ static inline void setColor(cairo_t* cr, const Color& col)
 
 static inline void setPlatformFill(GraphicsContext* context, cairo_t* cr, GraphicsContextPrivate* gcp)
 {
+    cairo_pattern_t* pattern = 0;
     cairo_save(cr);
     if (gcp->state.fillPattern) {
-        TransformationMatrix affine;
-        cairo_set_source(cr, gcp->state.fillPattern->createPlatformPattern(affine));
+        AffineTransform affine;
+        pattern = gcp->state.fillPattern->createPlatformPattern(affine);
+        cairo_set_source(cr, pattern);
     } else if (gcp->state.fillGradient)
         cairo_set_source(cr, gcp->state.fillGradient->platformGradient());
     else
@@ -87,14 +88,18 @@ static inline void setPlatformFill(GraphicsContext* context, cairo_t* cr, Graphi
     cairo_clip_preserve(cr);
     cairo_paint_with_alpha(cr, gcp->state.globalAlpha);
     cairo_restore(cr);
+    if (pattern)
+        cairo_pattern_destroy(pattern);
 }
 
 static inline void setPlatformStroke(GraphicsContext* context, cairo_t* cr, GraphicsContextPrivate* gcp)
 {
+    cairo_pattern_t* pattern = 0;
     cairo_save(cr);
     if (gcp->state.strokePattern) {
-        TransformationMatrix affine;
-        cairo_set_source(cr, gcp->state.strokePattern->createPlatformPattern(affine));
+        AffineTransform affine;
+        pattern = gcp->state.strokePattern->createPlatformPattern(affine);
+        cairo_set_source(cr, pattern);
     } else if (gcp->state.strokeGradient)
         cairo_set_source(cr, gcp->state.strokeGradient->platformGradient());
     else  {
@@ -108,6 +113,8 @@ static inline void setPlatformStroke(GraphicsContext* context, cairo_t* cr, Grap
     }
     cairo_stroke_preserve(cr);
     cairo_restore(cr);
+    if (pattern)
+        cairo_pattern_destroy(pattern);
 }
 
 // A fillRect helper
@@ -209,15 +216,7 @@ GraphicsContext::~GraphicsContext()
     delete m_data;
 }
 
-TransformationMatrix GraphicsContext::getCTM() const
-{
-    cairo_t* cr = platformContext();
-    cairo_matrix_t m;
-    cairo_get_matrix(cr, &m);
-    return TransformationMatrix(m.xx, m.yx, m.xy, m.yy, m.x0, m.y0);
-}
-
-AffineTransform GraphicsContext::getAffineCTM() const
+AffineTransform GraphicsContext::getCTM() const
 {
     cairo_t* cr = platformContext();
     cairo_matrix_t m;
@@ -800,17 +799,6 @@ void GraphicsContext::setURLForRect(const KURL& link, const IntRect& destRect)
     notImplemented();
 }
 
-void GraphicsContext::concatCTM(const TransformationMatrix& transform)
-{
-    if (paintingDisabled())
-        return;
-
-    cairo_t* cr = m_data->cr;
-    const cairo_matrix_t matrix = cairo_matrix_t(transform);
-    cairo_transform(cr, &matrix);
-    m_data->concatCTM(transform);
-}
-
 void GraphicsContext::concatCTM(const AffineTransform& transform)
 {
     if (paintingDisabled())
