@@ -58,10 +58,10 @@ static void clipboard_get_contents_cb(GtkClipboard *clipboard, GtkSelectionData
                                       guint info, gpointer data) {
     PasteboardSelectionData* clipboardData = reinterpret_cast<PasteboardSelectionData*>(data);
     ASSERT(clipboardData);
-    if ((gint)info == Pasteboard::generalPasteboard()->m_helper->getWebViewTargetInfoHtml())
+    if (info == Pasteboard::generalPasteboard()->helper()->getIdForTargetType(PasteboardHelper::TargetTypeMarkup))
         gtk_selection_data_set(selection_data, selection_data->target, 8,
                                reinterpret_cast<const guchar*>(clipboardData->markup()),
-                               g_utf8_strlen(clipboardData->markup(), -1));
+                               strlen(clipboardData->markup()));
     else
         gtk_selection_data_set_text(selection_data, clipboardData->text(), -1);
 }
@@ -89,6 +89,11 @@ Pasteboard::~Pasteboard()
     delete m_helper;
 }
 
+PasteboardHelper* Pasteboard::helper()
+{
+    return m_helper;
+}
+
 void Pasteboard::setHelper(PasteboardHelper* helper)
 {
     m_helper = helper;
@@ -121,7 +126,7 @@ void Pasteboard::writeURL(const KURL& url, const String&, Frame* frame)
         return;
 
     GtkClipboard* clipboard = m_helper->getClipboard(frame);
-    GtkClipboard* primary = m_helper->getPrimary(frame);
+    GtkClipboard* primary = m_helper->getPrimarySelectionClipboard(frame);
     CString utf8 = url.string().utf8();
     gtk_clipboard_set_text(clipboard, utf8.data(), utf8.length());
     gtk_clipboard_set_text(primary, utf8.data(), utf8.length());
@@ -161,7 +166,7 @@ PassRefPtr<DocumentFragment> Pasteboard::documentFragment(Frame* frame, PassRefP
                                                           bool allowPlainText, bool& chosePlainText)
 {
     GdkAtom textHtml = gdk_atom_intern_static_string("text/html");
-    GtkClipboard* clipboard = m_helper->getCurrentTarget(frame);
+    GtkClipboard* clipboard = m_helper->getCurrentClipboard(frame);
     chosePlainText = false;
 
     if (GtkSelectionData* data = gtk_clipboard_wait_for_contents(clipboard, textHtml)) {
@@ -196,7 +201,7 @@ PassRefPtr<DocumentFragment> Pasteboard::documentFragment(Frame* frame, PassRefP
 
 String Pasteboard::plainText(Frame* frame)
 {
-    GtkClipboard* clipboard = m_helper->getCurrentTarget(frame);
+    GtkClipboard* clipboard = m_helper->getCurrentClipboard(frame);
 
     gchar* utf8 = gtk_clipboard_wait_for_text(clipboard);
 
