@@ -64,7 +64,7 @@ static GdkVisual* getVisual(Widget* widget)
     }
 
 
-    return gdk_drawable_get_visual(GDK_DRAWABLE(container->window));
+    return gdk_drawable_get_visual(GDK_DRAWABLE(gtk_widget_get_window(container)));
 }
 
 int screenDepth(Widget* widget)
@@ -72,7 +72,7 @@ int screenDepth(Widget* widget)
     GdkVisual* visual = getVisual(widget);
     if (!visual)
         return 24;
-    return visual->depth;
+    return gdk_visual_get_depth(visual);
 }
 
 int screenDepthPerComponent(Widget* widget)
@@ -81,7 +81,7 @@ int screenDepthPerComponent(Widget* widget)
     if (!visual)
         return 8;
 
-    return visual->bits_per_rgb;
+    return gdk_visual_get_bits_per_rgb(visual);
 }
 
 bool screenIsMonochrome(Widget* widget)
@@ -102,7 +102,7 @@ FloatRect screenRect(Widget* widget)
     if (!screen)
         return FloatRect();
 
-    gint monitor = gdk_screen_get_monitor_at_window(screen, GTK_WIDGET(container)->window);
+    gint monitor = gdk_screen_get_monitor_at_window(screen, gtk_widget_get_window(GTK_WIDGET(container)));
     GdkRectangle geometry;
     gdk_screen_get_monitor_geometry(screen, monitor, &geometry);
     
