@@ -24,7 +24,7 @@
 #include "Cookie.h"
 #include "CString.h"
 #include "Document.h"
-#include "GOwnPtrGtk.h"
+#include "GOwnPtrSoup.h"
 #include "KURL.h"
 
 namespace WebCore {
@@ -36,7 +36,11 @@ SoupCookieJar* defaultCookieJar()
 {
     if (!cookiesInitialized) {
         cookiesInitialized = true;
-        setDefaultCookieJar(soup_cookie_jar_new());
+
+        cookieJar = soup_cookie_jar_new();
+#ifdef HAVE_LIBSOUP_2_29_90
+        soup_cookie_jar_set_accept_policy(cookieJar, SOUP_COOKIE_JAR_ACCEPT_NO_THIRD_PARTY);
+#endif
     }
 
     return cookieJar;
@@ -93,6 +97,22 @@ String cookies(const Document* /*document*/, const KURL& url)
     return result;
 }
 
+String cookieRequestHeaderFieldValue(const Document* /*document*/, const KURL& url)
+{
+    SoupCookieJar* jar = defaultCookieJar();
+    if (!jar)
+        return String();
+
+    SoupURI* uri = soup_uri_new(url.string().utf8().data());
+    char* cookies = soup_cookie_jar_get_cookies(jar, uri, TRUE);
+    soup_uri_free(uri);
+
+    String result(String::fromUTF8(cookies));
+    g_free(cookies);
+
+    return result;
+}
+
 bool cookiesEnabled(const Document* /*document*/)
 {
     return defaultCookieJar();
