@@ -1,6 +1,4 @@
 /*
- * This file is part of the html renderer for KDE.
- *
  * Copyright (C) 2005 Apple Computer
  *
  * This library is free software; you can redistribute it and/or
@@ -59,12 +57,14 @@ public:
 
     virtual bool canHaveChildren() const;
 
-protected:
+private:
     virtual void styleWillChange(StyleDifference, const RenderStyle* newStyle);
     virtual void styleDidChange(StyleDifference, const RenderStyle* oldStyle);
 
     virtual bool hasLineIfEmpty() const { return true; }
 
+    virtual bool requiresForcedStyleRecalcPropagation() const { return true; }
+
     void timerFired(Timer<RenderButton>*);
 
     RenderTextFragment* m_buttonText;
@@ -74,16 +74,16 @@ protected:
     bool m_default;
 };
 
-inline RenderButton* toRenderButton(RenderObject* o)
+inline RenderButton* toRenderButton(RenderObject* object)
 { 
-    ASSERT(!o || o->isRenderButton());
-    return static_cast<RenderButton*>(o);
+    ASSERT(!object || object->isRenderButton());
+    return static_cast<RenderButton*>(object);
 }
 
-inline const RenderButton* toRenderButton(const RenderObject* o)
+inline const RenderButton* toRenderButton(const RenderObject* object)
 { 
-    ASSERT(!o || o->isRenderButton());
-    return static_cast<const RenderButton*>(o);
+    ASSERT(!object || object->isRenderButton());
+    return static_cast<const RenderButton*>(object);
 }
 
 // This will catch anyone doing an unnecessary cast.
