@@ -35,6 +35,7 @@ public:
     RenderFrame(HTMLFrameElement*);
 
     FrameEdgeInfo edgeInfo() const;
+    void layoutWithFlattening(bool fixedWidth, bool fixedHeight);
 
 private:
     virtual const char* renderName() const { return "RenderFrame"; }
@@ -43,6 +44,15 @@ private:
     virtual void viewCleared();
 };
 
+inline RenderFrame* toRenderFrame(RenderObject* object)
+{
+    ASSERT(!object || object->isFrame());
+    return static_cast<RenderFrame*>(object);
+}
+
+// This will catch anyone doing an unnecessary cast.
+void toRenderFrame(const RenderFrame*);
+
 } // namespace WebCore
 
 #endif // RenderFrame_h
