@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 1999 Lars Knoll (knoll@kde.org)
  *           (C) 1999 Antti Koivisto (koivisto@kde.org)
- * Copyright (C) 2003, 2004, 2005, 2006, 2007 Apple Inc. All rights reserved.
+ * Copyright (C) 2003, 2004, 2005, 2006, 2007, 2009 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -36,8 +36,15 @@ String listMarkerText(EListStyleType, int value);
 class RenderListMarker : public RenderBox {
 public:
     RenderListMarker(RenderListItem*);
-    ~RenderListMarker();
+    virtual ~RenderListMarker();
 
+    virtual void calcPrefWidths();
+
+    const String& text() const { return m_text; }
+
+    bool isInside() const;
+
+private:
     virtual const char* renderName() const { return "RenderListMarker"; }
 
     virtual bool isListMarker() const { return true; }
@@ -45,7 +52,6 @@ public:
     virtual void paint(PaintInfo&, int tx, int ty);
 
     virtual void layout();
-    virtual void calcPrefWidths();
 
     virtual void imageChanged(WrappedImagePtr, const IntRect* = 0);
 
@@ -56,9 +62,6 @@ public:
 
     bool isImage() const;
     bool isText() const { return !isImage(); }
-    const String& text() const { return m_text; }
-
-    bool isInside() const;
 
     virtual void setSelectionState(SelectionState);
     virtual IntRect selectionRectForRepaint(RenderBoxModelObject* repaintContainer, bool clipToVisibleContent = true);
@@ -66,18 +69,32 @@ public:
 
     void updateMargins();
 
-protected:
     virtual void styleWillChange(StyleDifference, const RenderStyle* newStyle);
     virtual void styleDidChange(StyleDifference, const RenderStyle* oldStyle);
 
-private:
     IntRect getRelativeMarkerRect();
+    IntRect localSelectionRect();
 
     String m_text;
     RefPtr<StyleImage> m_image;
     RenderListItem* m_listItem;
 };
 
+inline RenderListMarker* toRenderListMarker(RenderObject* object)
+{
+    ASSERT(!object || object->isListMarker());
+    return static_cast<RenderListMarker*>(object);
+}
+
+inline const RenderListMarker* toRenderListMarker(const RenderObject* object)
+{
+    ASSERT(!object || object->isListMarker());
+    return static_cast<const RenderListMarker*>(object);
+}
+
+// This will catch anyone doing an unnecessary cast.
+void toRenderListMarker(const RenderListMarker*);
+
 } // namespace WebCore
 
 #endif // RenderListMarker_h
