@@ -103,7 +103,7 @@ double SliderRange::clampValue(double value)
 double SliderRange::valueFromElement(HTMLInputElement* element, bool* wasClamped)
 {
     double oldValue;
-    bool parseSuccess = HTMLInputElement::formStringToDouble(element->value(), &oldValue);
+    bool parseSuccess = HTMLInputElement::parseToDoubleForNumberType(element->value(), &oldValue);
     if (!parseSuccess)
         oldValue = (minimum + maximum) / 2;
     double newValue = clampValue(oldValue);
@@ -379,7 +379,7 @@ void RenderSlider::updateFromElement()
     bool clamped;
     double value = range.valueFromElement(element, &clamped);
     if (clamped)
-        element->setValueFromRenderer(HTMLInputElement::formStringFromDouble(value));
+        element->setValueFromRenderer(HTMLInputElement::serializeForNumberType(value));
 
     // Layout will take care of the thumb's size and position.
     if (!m_thumb) {
@@ -435,7 +435,7 @@ void RenderSlider::setValueForPosition(int position)
     if (style()->appearance() == SliderVerticalPart || style()->appearance() == MediaVolumeSliderPart)
         fraction = 1 - fraction;
     double value = range.clampValue(range.valueFromProportion(fraction));
-    element->setValueFromRenderer(HTMLInputElement::formStringFromDouble(value));
+    element->setValueFromRenderer(HTMLInputElement::serializeForNumberType(value));
 
     // Also update the position if appropriate.
     if (position != currentPosition()) {
