@@ -97,6 +97,11 @@ void RenderView::calcPrefWidths()
     m_maxPrefWidth = m_minPrefWidth;
 }
 
+bool RenderView::isChildAllowed(RenderObject* child, RenderStyle*) const
+{
+    return child->isBox();
+}
+
 void RenderView::layout()
 {
     if (printing())
@@ -541,6 +546,8 @@ void RenderView::updateWidgetPositions()
     RenderWidgetSet::iterator end = m_widgets.end();
     for (RenderWidgetSet::iterator it = m_widgets.begin(); it != end; ++it)
         (*it)->updateWidgetPosition();
+    for (RenderWidgetSet::iterator it = m_widgets.begin(); it != end; ++it)
+        (*it)->widgetPositionsUpdated();
 }
 
 void RenderView::addWidget(RenderWidget* o)
