@@ -22,6 +22,7 @@
 #include "config.h"
 #include "RenderStyle.h"
 
+#include "CSSPropertyNames.h"
 #include "CSSStyleSelector.h"
 #include "CachedImage.h"
 #include "CounterContent.h"
@@ -58,8 +59,7 @@ PassRefPtr<RenderStyle> RenderStyle::clone(const RenderStyle* other)
 }
 
 RenderStyle::RenderStyle()
-    : m_pseudoState(PseudoUnknown)
-    , m_affectedByAttributeSelectors(false)
+    : m_affectedByAttributeSelectors(false)
     , m_unique(false)
     , m_affectedByEmpty(false)
     , m_emptyState(false)
@@ -86,8 +86,7 @@ RenderStyle::RenderStyle()
 }
 
 RenderStyle::RenderStyle(bool)
-    : m_pseudoState(PseudoUnknown)
-    , m_affectedByAttributeSelectors(false)
+    : m_affectedByAttributeSelectors(false)
     , m_unique(false)
     , m_affectedByEmpty(false)
     , m_emptyState(false)
@@ -121,7 +120,6 @@ RenderStyle::RenderStyle(bool)
 
 RenderStyle::RenderStyle(const RenderStyle& o)
     : RefCounted<RenderStyle>()
-    , m_pseudoState(o.m_pseudoState)
     , m_affectedByAttributeSelectors(false)
     , m_unique(false)
     , m_affectedByEmpty(false)
@@ -192,6 +190,11 @@ static inline int pseudoBit(PseudoId pseudo)
     return 1 << (pseudo - 1);
 }
 
+bool RenderStyle::hasAnyPublicPseudoStyles() const
+{
+    return PUBLIC_PSEUDOID_MASK & noninherited_flags._pseudoBits;
+}
+
 bool RenderStyle::hasPseudoStyle(PseudoId pseudo) const
 {
     ASSERT(pseudo > NOPSEUDO);
@@ -208,8 +211,17 @@ void RenderStyle::setHasPseudoStyle(PseudoId pseudo)
 
 RenderStyle* RenderStyle::getCachedPseudoStyle(PseudoId pid) const
 {
-    if (!m_cachedPseudoStyle || styleType() != NOPSEUDO)
+    ASSERT(styleType() != VISITED_LINK);
+
+    if (!m_cachedPseudoStyle)
         return 0;
+
+    if (styleType() != NOPSEUDO) {
+        if (pid == VISITED_LINK)
+            return m_cachedPseudoStyle->styleType() == VISITED_LINK ? m_cachedPseudoStyle.get() : 0;
+        return 0;
+    }
+
     RenderStyle* ps = m_cachedPseudoStyle.get();
     while (ps && ps->styleType() != pid)
         ps = ps->m_cachedPseudoStyle.get();
@@ -324,6 +336,7 @@ StyleDifference RenderStyle::diff(const RenderStyle* other, unsigned& changedCon
             *rareNonInheritedData->flexibleBox.get() != *other->rareNonInheritedData->flexibleBox.get())
             return StyleDifferenceLayout;
 
+        // FIXME: We should add an optimized form of layout that just recomputes visual overflow.
         if (!rareNonInheritedData->shadowDataEquivalent(*other->rareNonInheritedData.get()))
             return StyleDifferenceLayout;
 
@@ -453,6 +466,12 @@ StyleDifference RenderStyle::diff(const RenderStyle* other, unsigned& changedCon
     if (inherited->m_effectiveZoom != other->inherited->m_effectiveZoom)
         return StyleDifferenceLayout;
 
+    if ((rareNonInheritedData->opacity == 1 && other->rareNonInheritedData->opacity < 1) ||
+        (rareNonInheritedData->opacity < 1 && other->rareNonInheritedData->opacity == 1)) {
+        // FIXME: We should add an optimized form of layout that just recomputes visual overflow.
+        return StyleDifferenceLayout;
+    }
+
     // Make sure these left/top/right/bottom checks stay below all layout checks and above
     // all visible checks.
     if (position() != StaticPosition) {
@@ -490,6 +509,7 @@ StyleDifference RenderStyle::diff(const RenderStyle* other, unsigned& changedCon
         inherited_flags._visibility != other->inherited_flags._visibility ||
         inherited_flags._text_decorations != other->inherited_flags._text_decorations ||
         inherited_flags._force_backgrounds_to_white != other->inherited_flags._force_backgrounds_to_white ||
+        inherited_flags._insideLink != other->inherited_flags._insideLink ||
         surround->border != other->surround->border ||
         *background.get() != *other->background.get() ||
         visual->textDecoration != other->visual->textDecoration ||
@@ -695,7 +715,7 @@ void RenderStyle::addBindingURI(StringImpl* uri)
 
 void RenderStyle::setTextShadow(ShadowData* val, bool add)
 {
-    ASSERT(!val || !val->spread);
+    ASSERT(!val || (!val->spread && val->style == Normal));
 
     StyleRareInheritedData* rareData = rareInheritedData.access();
     if (!add) {
@@ -906,4 +926,147 @@ void RenderStyle::setBlendedFontSize(int size)
     font().update(font().fontSelector());
 }
 
+void RenderStyle::getBoxShadowExtent(int &top, int &right, int &bottom, int &left) const
+{
+    top = 0;
+    right = 0;
+    bottom = 0;
+    left = 0;
+
+    for (ShadowData* boxShadow = this->boxShadow(); boxShadow; boxShadow = boxShadow->next) {
+        if (boxShadow->style == Inset)
+            continue;
+        int blurAndSpread = boxShadow->blur + boxShadow->spread;
+
+        top = min(top, boxShadow->y - blurAndSpread);
+        right = max(right, boxShadow->x + blurAndSpread);
+        bottom = max(bottom, boxShadow->y + blurAndSpread);
+        left = min(left, boxShadow->x - blurAndSpread);
+    }
+}
+
+void RenderStyle::getBoxShadowHorizontalExtent(int &left, int &right) const
+{
+    left = 0;
+    right = 0;
+
+    for (ShadowData* boxShadow = this->boxShadow(); boxShadow; boxShadow = boxShadow->next) {
+        if (boxShadow->style == Inset)
+            continue;
+        int blurAndSpread = boxShadow->blur + boxShadow->spread;
+
+        left = min(left, boxShadow->x - blurAndSpread);
+        right = max(right, boxShadow->x + blurAndSpread);
+    }
+}
+
+void RenderStyle::getBoxShadowVerticalExtent(int &top, int &bottom) const
+{
+    top = 0;
+    bottom = 0;
+
+    for (ShadowData* boxShadow = this->boxShadow(); boxShadow; boxShadow = boxShadow->next) {
+        if (boxShadow->style == Inset)
+            continue;
+        int blurAndSpread = boxShadow->blur + boxShadow->spread;
+
+        top = min(top, boxShadow->y - blurAndSpread);
+        bottom = max(bottom, boxShadow->y + blurAndSpread);
+    }
+}
+
+static EBorderStyle borderStyleForColorProperty(const RenderStyle* style, int colorProperty)
+{
+    EBorderStyle borderStyle;
+    switch (colorProperty) {
+    case CSSPropertyBorderLeftColor:
+        borderStyle = style->borderLeftStyle();
+        break;
+    case CSSPropertyBorderRightColor:
+        borderStyle = style->borderRightStyle();
+        break;
+    case CSSPropertyBorderTopColor:
+        borderStyle = style->borderTopStyle();
+        break;
+    case CSSPropertyBorderBottomColor:
+        borderStyle = style->borderBottomStyle();
+        break;
+    default:
+        borderStyle = BNONE;
+        break;
+    }
+    return borderStyle;
+}
+
+static Color colorIncludingFallback(const RenderStyle* style, int colorProperty, EBorderStyle borderStyle)
+{
+    Color result;
+    switch (colorProperty) {
+    case CSSPropertyBackgroundColor:
+        return style->backgroundColor(); // Background color doesn't fall back.
+    case CSSPropertyBorderLeftColor:
+        result = style->borderLeftColor();
+        borderStyle = style->borderLeftStyle();
+        break;
+    case CSSPropertyBorderRightColor:
+        result = style->borderRightColor();
+        borderStyle = style->borderRightStyle();
+        break;
+    case CSSPropertyBorderTopColor:
+        result = style->borderTopColor();
+        borderStyle = style->borderTopStyle();
+        break;
+    case CSSPropertyBorderBottomColor:
+        result = style->borderBottomColor();
+        borderStyle = style->borderBottomStyle();
+        break;
+    case CSSPropertyColor:
+        result = style->color();
+        break;
+    case CSSPropertyOutlineColor:
+        result = style->outlineColor();
+        break;
+    case CSSPropertyWebkitColumnRuleColor:
+        result = style->columnRuleColor();
+        break;
+    case CSSPropertyWebkitTextFillColor:
+        result = style->textFillColor();
+        break;
+    case CSSPropertyWebkitTextStrokeColor:
+        result = style->textStrokeColor();
+        break;
+    default:
+        // FIXME: Add SVG fill and stroke.
+        ASSERT_NOT_REACHED();
+        break;
+    }
+
+    if (!result.isValid()) {
+        if ((colorProperty == CSSPropertyBorderLeftColor || colorProperty == CSSPropertyBorderRightColor
+            || colorProperty == CSSPropertyBorderTopColor || colorProperty == CSSPropertyBorderBottomColor)
+            && (borderStyle == INSET || borderStyle == OUTSET || borderStyle == RIDGE || borderStyle == GROOVE))
+            result.setRGB(238, 238, 238);
+        else
+            result = style->color();
+    }
+
+    return result;
+}
+
+Color RenderStyle::visitedDependentColor(int colorProperty) const
+{
+    EBorderStyle borderStyle = borderStyleForColorProperty(this, colorProperty);
+    Color unvisitedColor = colorIncludingFallback(this, colorProperty, borderStyle);
+    if (insideLink() != InsideVisitedLink)
+        return unvisitedColor;
+
+    RenderStyle* visitedStyle = getCachedPseudoStyle(VISITED_LINK);
+    if (!visitedStyle)
+        return unvisitedColor;
+    Color visitedColor = colorIncludingFallback(visitedStyle, colorProperty, borderStyle);
+
+    // Take the alpha from the unvisited color, but get the RGB values from the visited color.
+    return Color(visitedColor.red(), visitedColor.green(), visitedColor.blue(), unvisitedColor.alpha());
+}
+
 } // namespace WebCore
