@@ -28,6 +28,8 @@
 #ifndef DatabaseAuthorizer_h
 #define DatabaseAuthorizer_h
 
+#include "StringHash.h"
+#include <wtf/HashSet.h>
 #include <wtf/PassRefPtr.h>
 #include <wtf/Threading.h>
 
@@ -59,10 +61,10 @@ public:
     int dropTrigger(const String& triggerName, const String& tableName);
     int dropTempTrigger(const String& triggerName, const String& tableName);
 
-    int createView(const String& /*viewName*/) { return SQLAuthAllow; }
-    int createTempView(const String& /*viewName*/) { return SQLAuthAllow; }
-    int dropView(const String& /*viewName*/) { return SQLAuthAllow; }
-    int dropTempView(const String& /*viewName*/) { return SQLAuthAllow; }
+    int createView(const String& viewName);
+    int createTempView(const String& viewName);
+    int dropView(const String& viewName);
+    int dropTempView(const String& viewName);
 
     int createVTable(const String& tableName, const String& moduleName);
     int dropVTable(const String& tableName, const String& moduleName);
@@ -75,7 +77,7 @@ public:
     int allowSelect() { return SQLAuthAllow; }
     int allowRead(const String& tableName, const String& columnName);
 
-    int allowReindex(const String& /*indexName*/) { return SQLAuthAllow; }
+    int allowReindex(const String& indexName);
     int allowAnalyze(const String& tableName);
     int allowFunction(const String& functionName);
     int allowPragma(const String& pragmaName, const String& firstArgument);
@@ -94,12 +96,15 @@ public:
 
 private:
     DatabaseAuthorizer();
+    void addWhitelistedFunctions();
     int denyBasedOnTableName(const String&);
 
     bool m_securityEnabled : 1;
     bool m_lastActionWasInsert : 1;
     bool m_lastActionChangedDatabase : 1;
     bool m_readOnly : 1;
+
+    HashSet<String, CaseFoldingHash> m_whitelistedFunctions;
 };
 
 } // namespace WebCore
