@@ -2,8 +2,6 @@
     Copyright (C) 2004, 2005, 2008 Nikolas Zimmermann <zimmermann@kde.org>
                   2004, 2005, 2006 Rob Buis <buis@kde.org>
 
-    This file is part of the KDE project
-
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
@@ -30,7 +28,7 @@
 namespace WebCore {
 
 SVGNumberList::SVGNumberList(const QualifiedName& attributeName)
-    : SVGList<float>(attributeName)
+    : SVGPODList<float>(attributeName)
 {
 }
 
