@@ -2,8 +2,6 @@
     Copyright (C) 2004, 2005, 2006, 2008 Nikolas Zimmermann <zimmermann@kde.org>
                   2004, 2005 Rob Buis <buis@kde.org>
 
-    This file is part of the KDE project
-
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
     License as published by the Free Software Foundation; either
@@ -23,8 +21,8 @@
 #include "config.h"
 
 #if ENABLE(SVG)
-
 #include "SVGPointList.h"
+#include "PlatformString.h"
 
 namespace WebCore {
 
@@ -37,6 +35,24 @@ SVGPointList::~SVGPointList()
 {
 }
 
+String SVGPointList::valueAsString() const
+{
+    String result;
+
+    ExceptionCode ec = 0;
+    for (unsigned int i = 0; i < numberOfItems(); ++i) {
+        if (i > 0)
+            result += " ";
+
+        FloatPoint point = getItem(i, ec);
+        ASSERT(ec == 0);
+
+        result += String::format("%.6lg %.6lg", point.x(), point.y());
+    }
+
+    return result;
+}
+
 }
 
 #endif // ENABLE(SVG)
