@@ -79,7 +79,7 @@ SVGPaintServer* SVGPaintServer::fillPaintServer(const RenderStyle* style, const
         return 0;
 
     SVGPaint* fill = style->svgStyle()->fillPaint();
-
+        
     SVGPaintServer* fillPaintServer = 0;
     SVGPaint::SVGPaintType paintType = fill->paintType();
     if (paintType == SVGPaint::SVG_PAINTTYPE_URI ||
@@ -98,10 +98,29 @@ SVGPaintServer* SVGPaintServer::fillPaintServer(const RenderStyle* style, const
     if (paintType != SVGPaint::SVG_PAINTTYPE_URI && !fillPaintServer) {
         fillPaintServer = sharedSolidPaintServer();
         SVGPaintServerSolid* fillPaintServerSolid = static_cast<SVGPaintServerSolid*>(fillPaintServer);
+        
+        Color fillColor;
         if (paintType == SVGPaint::SVG_PAINTTYPE_CURRENTCOLOR)
-            fillPaintServerSolid->setColor(style->color());
+            fillColor = style->color();
         else
-            fillPaintServerSolid->setColor(fill->color());
+            fillColor = fill->color();
+
+        if (style->insideLink() == InsideVisitedLink) {
+            RenderStyle* visitedStyle = style->getCachedPseudoStyle(VISITED_LINK);
+            SVGPaint* visitedFill = visitedStyle->svgStyle()->fillPaint();
+            Color visitedFillColor;
+            if (visitedFill->paintType() != SVGPaint::SVG_PAINTTYPE_URI) {
+                if (visitedFill->paintType() == SVGPaint::SVG_PAINTTYPE_CURRENTCOLOR)
+                    visitedFillColor = visitedStyle->color();
+                else
+                    visitedFillColor = visitedFill->color();
+                if (visitedFillColor.isValid())
+                    fillColor = Color(visitedFillColor.red(), visitedFillColor.green(), visitedFillColor.blue(), fillColor.alpha());
+            }
+        }
+
+        fillPaintServerSolid->setColor(fillColor);
+        
         // FIXME: Ideally invalid colors would never get set on the RenderStyle and this could turn into an ASSERT
         if (!fillPaintServerSolid->color().isValid())
             fillPaintServer = 0;
@@ -123,8 +142,10 @@ SVGPaintServer* SVGPaintServer::strokePaintServer(const RenderStyle* style, cons
 
     SVGPaintServer* strokePaintServer = 0;
     SVGPaint::SVGPaintType paintType = stroke->paintType();
-    if (paintType == SVGPaint::SVG_PAINTTYPE_URI ||
-        paintType == SVGPaint::SVG_PAINTTYPE_URI_RGBCOLOR) {
+    if ((paintType == SVGPaint::SVG_PAINTTYPE_URI
+        || paintType == SVGPaint::SVG_PAINTTYPE_URI_RGBCOLOR)
+        && item->objectBoundingBox().width() != 0
+        && item->objectBoundingBox().height() != 0) {
         AtomicString id(SVGURIReference::getTarget(stroke->uri()));
         strokePaintServer = getPaintServerById(item->document(), id, item);
 
@@ -139,14 +160,38 @@ SVGPaintServer* SVGPaintServer::strokePaintServer(const RenderStyle* style, cons
     if (paintType != SVGPaint::SVG_PAINTTYPE_URI && !strokePaintServer) {
         strokePaintServer = sharedSolidPaintServer();
         SVGPaintServerSolid* strokePaintServerSolid = static_cast<SVGPaintServerSolid*>(strokePaintServer);
+        
+        Color strokeColor;
         if (paintType == SVGPaint::SVG_PAINTTYPE_CURRENTCOLOR)
-            strokePaintServerSolid->setColor(style->color());
+            strokeColor = style->color();
         else
-            strokePaintServerSolid->setColor(stroke->color());
+            strokeColor = stroke->color();
+
+        if (style->insideLink() == InsideVisitedLink) {
+            RenderStyle* visitedStyle = style->getCachedPseudoStyle(VISITED_LINK);
+            SVGPaint* visitedStroke = visitedStyle->svgStyle()->strokePaint();
+            Color visitedStrokeColor;
+            if (visitedStroke->paintType() != SVGPaint::SVG_PAINTTYPE_URI) {
+                if (visitedStroke->paintType() == SVGPaint::SVG_PAINTTYPE_CURRENTCOLOR)
+                    visitedStrokeColor = visitedStyle->color();
+                else
+                    visitedStrokeColor = visitedStroke->color();
+                if (visitedStrokeColor.isValid())
+                    strokeColor = Color(visitedStrokeColor.red(), visitedStrokeColor.green(), visitedStrokeColor.blue(), strokeColor.alpha());
+            }
+        }
+
+        strokePaintServerSolid->setColor(strokeColor);
+
         // FIXME: Ideally invalid colors would never get set on the RenderStyle and this could turn into an ASSERT
         if (!strokePaintServerSolid->color().isValid())
             strokePaintServer = 0;
     }
+    if (!strokePaintServer) {
+        // default value (black), see bug 11017
+        strokePaintServer = sharedSolidPaintServer();
+        static_cast<SVGPaintServerSolid*>(strokePaintServer)->setColor(Color::black);
+    }
 
     return strokePaintServer;
 }
