@@ -1,6 +1,7 @@
 /*
  * Copyright (C) 2007 Luca Bruno <lethalman88@gmail.com>
  * Copyright (C) 2009 Holger Hans Peter Freyther
+ * Copyright (C) 2010 Martin Robinson <mrobinson@webkit.org>
  * All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
@@ -39,13 +40,12 @@ namespace WebKit {
 
 class PasteboardHelperGtk : public PasteboardHelper {
 public:
-    PasteboardHelperGtk() { }
-    virtual GtkClipboard* getCurrentTarget(Frame*) const;
-    virtual GtkClipboard* getClipboard(Frame*) const;
-    virtual GtkClipboard* getPrimary(Frame*) const;
-    virtual GtkTargetList* getCopyTargetList(Frame*) const;
-    virtual GtkTargetList* getPasteTargetList(Frame*) const;
-    virtual gint getWebViewTargetInfoHtml() const;
+    PasteboardHelperGtk();
+    ~PasteboardHelperGtk();
+    virtual guint getIdForTargetType(PasteboardTargetType);
+
+protected:
+    virtual bool usePrimarySelectionClipboard(GtkWidget*);
 };
 
 }
