@@ -45,6 +45,7 @@
 #include <glib/gi18n-lib.h>
 #include "GCController.h"
 #include "GraphicsContext.h"
+#include "GtkVersioning.h"
 #include "HTMLFrameOwnerElement.h"
 #include "JSDOMWindow.h"
 #include "JSLock.h"
@@ -54,6 +55,9 @@
 #include "JSDOMBinding.h"
 #include "ScriptController.h"
 #include "SubstituteData.h"
+#if ENABLE(SVG)
+#include "SVGSMILElement.h"
+#endif
 
 #include <atk/atk.h>
 #include <JavaScriptCore/APICast.h>
@@ -181,6 +185,14 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
             g_cclosure_marshal_VOID__VOID,
             G_TYPE_NONE, 0);
 
+    /**
+     * WebKitWebFrame::load-done
+     * @web_frame: the object on which the signal is emitted
+     *
+     * Emitted when frame loading is done.
+     *
+     * Deprecated: Use the "load-status" property instead.
+     */
     webkit_web_frame_signals[LOAD_COMMITTED] = g_signal_new("load-committed",
             G_TYPE_FROM_CLASS(frameClass),
             (GSignalFlags)G_SIGNAL_RUN_LAST,
@@ -196,7 +208,7 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
      *
      * Emitted when frame loading is done.
      *
-     * Deprecated: Use WebKitWebView::load-finished instead, and/or
+     * Deprecated: Use the "load-status" property instead, and/or
      * WebKitWebView::load-error to be notified of load errors
      */
     webkit_web_frame_signals[LOAD_DONE] = g_signal_new("load-done",
@@ -209,6 +221,15 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
             G_TYPE_NONE, 1,
             G_TYPE_BOOLEAN);
 
+    /**
+     * WebKitWebFrame::title-changed:
+     * @frame: the object on which the signal is emitted
+     * @title: the new title
+     *
+     * When a #WebKitWebFrame changes the document title this signal is emitted.
+     *
+     * Deprecated: 1.1.18: Use "notify::title" instead.
+     */
     webkit_web_frame_signals[TITLE_CHANGED] = g_signal_new("title-changed",
             G_TYPE_FROM_CLASS(frameClass),
             (GSignalFlags)G_SIGNAL_RUN_LAST,
@@ -249,6 +270,9 @@ static void webkit_web_frame_class_init(WebKitWebFrameClass* frameClass)
      * WebKitWebFrame:horizontal-scrollbar-policy and
      * WebKitWebFrame:vertical-scrollbar-policy properties.
      *
+     * Return value: %TRUE to stop other handlers from being invoked for the
+     * event. %FALSE to propagate the event further.
+     *
      * Since: 1.1.14
      */
     webkit_web_frame_signals[SCROLLBARS_POLICY_CHANGED] = g_signal_new("scrollbars-policy-changed",
@@ -794,10 +818,74 @@ gchar* webkit_web_frame_dump_render_tree(WebKitWebFrame* frame)
     if (view && view->layoutPending())
         view->layout();
 
-    String string = externalRepresentation(coreFrame->contentRenderer());
+    String string = externalRepresentation(coreFrame);
     return g_strdup(string.utf8().data());
 }
 
+/**
+ * webkit_web_frame_counter_value_for_element_by_id:
+ * @frame: a #WebKitWebFrame
+ * @id: an element ID string
+ *
+ * Return value: The counter value of element @id in @frame
+ */
+gchar* webkit_web_frame_counter_value_for_element_by_id(WebKitWebFrame* frame, const gchar* id)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
+
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return 0;
+
+    Element* coreElement = coreFrame->document()->getElementById(AtomicString(id));
+    if (!coreElement)
+        return 0;
+    String counterValue = counterValueForElement(coreElement);
+    return g_strdup(counterValue.utf8().data());
+}
+
+/**
+ * webkit_web_frame_page_number_for_element_by_id
+ * @frame: a #WebKitWebFrame
+ * @id: an element ID string
+ * @pageWidth: width of a page
+ * @pageHeight: height of a page
+ *
+ * Return value: The number of page where the specified element will be put
+ */
+int webkit_web_frame_page_number_for_element_by_id(WebKitWebFrame* frame, const gchar* id, float pageWidth, float pageHeight)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
+
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return -1;
+
+    Element* coreElement = coreFrame->document()->getElementById(AtomicString(id));
+    if (!coreElement)
+        return -1;
+    return PrintContext::pageNumberForElement(coreElement, FloatSize(pageWidth, pageHeight));
+}
+
+/**
+ * webkit_web_frame_number_of_pages
+ * @frame: a #WebKitWebFrame
+ * @pageWidth: width of a page
+ * @pageHeight: height of a page
+ *
+ * Return value: The number of pages to be printed.
+ */
+int webkit_web_frame_number_of_pages(WebKitWebFrame* frame, float pageWidth, float pageHeight)
+{
+    g_return_val_if_fail(WEBKIT_IS_WEB_FRAME(frame), NULL);
+
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return -1;
+
+    return PrintContext::numberOfPages(coreFrame, FloatSize(pageWidth, pageHeight));
+}
+
 /**
  * webkit_web_frame_get_pending_unload_event_count:
  * @frame: a #WebKitWebFrame
@@ -855,7 +943,7 @@ static void end_print_callback(GtkPrintOperation* op, GtkPrintContext* context,
  * @action: the #GtkPrintOperationAction to be performed
  * @error: #GError for error return
  *
- * Prints the given #WebKitFrame, using the given #GtkPrintOperation
+ * Prints the given #WebKitWebFrame, using the given #GtkPrintOperation
  * and #GtkPrintOperationAction. This function wraps a call to
  * gtk_print_operation_run() for printing the contents of the
  * #WebKitWebFrame.
@@ -868,7 +956,8 @@ GtkPrintOperationResult webkit_web_frame_print_full(WebKitWebFrame* frame, GtkPr
     g_return_val_if_fail(GTK_IS_PRINT_OPERATION(operation), GTK_PRINT_OPERATION_RESULT_ERROR);
 
     GtkWidget* topLevel = gtk_widget_get_toplevel(GTK_WIDGET(webkit_web_frame_get_web_view(frame)));
-    if (!GTK_WIDGET_TOPLEVEL(topLevel))
+
+    if (!gtk_widget_is_toplevel(topLevel))
         topLevel = NULL;
 
     Frame* coreFrame = core(frame);
@@ -888,7 +977,7 @@ GtkPrintOperationResult webkit_web_frame_print_full(WebKitWebFrame* frame, GtkPr
  * webkit_web_frame_print:
  * @frame: a #WebKitWebFrame
  *
- * Prints the given #WebKitFrame, by presenting a print dialog to the
+ * Prints the given #WebKitWebFrame, by presenting a print dialog to the
  * user. If you need more control over the printing process, see
  * webkit_web_frame_print_full().
  *
@@ -907,11 +996,12 @@ void webkit_web_frame_print(WebKitWebFrame* frame)
 
     if (error) {
         GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(priv->webView));
-        GtkWidget* dialog = gtk_message_dialog_new(GTK_WIDGET_TOPLEVEL(window) ? GTK_WINDOW(window) : 0,
+        GtkWidget* dialog = gtk_message_dialog_new(gtk_widget_is_toplevel(window) ? GTK_WINDOW(window) : 0,
                                                    GTK_DIALOG_DESTROY_WITH_PARENT,
                                                    GTK_MESSAGE_ERROR,
                                                    GTK_BUTTONS_CLOSE,
                                                    "%s", error->message);
+
         g_error_free(error);
 
         g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
@@ -937,6 +1027,22 @@ bool webkit_web_frame_pause_transition(WebKitWebFrame* frame, const gchar* name,
     return core(frame)->animation()->pauseTransitionAtTime(coreElement->renderer(), AtomicString(name), time);
 }
 
+bool webkit_web_frame_pause_svg_animation(WebKitWebFrame* frame, const gchar* animationId, double time, const gchar* elementId)
+{
+    ASSERT(core(frame));
+#if ENABLE(SVG)
+    Document* document = core(frame)->document();
+    if (!document || !document->svgExtensions())
+        return false;
+    Element* coreElement = document->getElementById(AtomicString(animationId));
+    if (!coreElement || !SVGSMILElement::isSMILElement(coreElement))
+        return false;
+    return document->accessSVGExtensions()->sampleAnimationAtTime(elementId, static_cast<SVGSMILElement*>(coreElement), time);
+#else
+    return false;
+#endif
+}
+
 unsigned int webkit_web_frame_number_of_active_animations(WebKitWebFrame* frame)
 {
     Frame* coreFrame = core(frame);
@@ -1102,3 +1208,29 @@ void webkit_web_frame_layout(WebKitWebFrame* frame)
 
     view->layout();
 }
+
+/**
+ * webkit_web_frame_get_network_response:
+ * @frame: a #WebKitWebFrame
+ *
+ * Returns a #WebKitNetworkResponse object representing the response
+ * that was given to the request for the given frame, or NULL if the
+ * frame was not created by a load. You must unref the object when you
+ * are done with it.
+ *
+ * Return value: a #WebKitNetworkResponse object
+ *
+ * Since: 1.1.18
+ */
+WebKitNetworkResponse* webkit_web_frame_get_network_response(WebKitWebFrame* frame)
+{
+    Frame* coreFrame = core(frame);
+    if (!coreFrame)
+        return NULL;
+
+    WebCore::DocumentLoader* loader = coreFrame->loader()->activeDocumentLoader();
+    if (!loader)
+        return NULL;
+
+    return webkit_network_response_new_with_core_response(loader->response());
+}
