@@ -45,28 +45,27 @@
 #include <gdk/gdkkeysyms.h>
 #include <string.h>
 
-// TODO: Currently drag and drop related code is left out and
-// should be merged once we have drag and drop support in WebCore.
+// FIXME: Implement support for synthesizing drop events.
 
 extern "C" {
     extern void webkit_web_frame_layout(WebKitWebFrame* frame);
 }
 
-static bool down = false;
-static bool currentEventButton = 1;
-static bool dragMode = true;
-static bool replayingSavedEvents = false;
+static bool dragMode;
+static int timeOffset = 0;
+
 static int lastMousePositionX;
 static int lastMousePositionY;
-
 static int lastClickPositionX;
 static int lastClickPositionY;
-static int clickCount = 0;
+static int lastClickTimeOffset;
+static int lastClickButton;
+static int buttonCurrentlyDown;
+static int clickCount;
 
 struct DelayedMessage {
     GdkEvent event;
     gulong delay;
-    gboolean isDragEvent;
 };
 
 static DelayedMessage msgQueue[1024];
@@ -76,6 +75,18 @@ static unsigned startOfQueue;
 
 static const float zoomMultiplierRatio = 1.2f;
 
+// Key event location code defined in DOM Level 3.
+enum KeyLocationCode {
+    DOM_KEY_LOCATION_STANDARD      = 0x00,
+    DOM_KEY_LOCATION_LEFT          = 0x01,
+    DOM_KEY_LOCATION_RIGHT         = 0x02,
+    DOM_KEY_LOCATION_NUMPAD        = 0x03
+};
+
+static void sendOrQueueEvent(GdkEvent event);
+static void dispatchEvent(GdkEvent event);
+static guint getStateFlags();
+
 static JSValueRef getDragModeCallback(JSContextRef context, JSObjectRef object, JSStringRef propertyName, JSValueRef* exception)
 {
     return JSValueMakeBoolean(context, dragMode);
@@ -89,41 +100,67 @@ static bool setDragModeCallback(JSContextRef context, JSObjectRef object, JSStri
 
 static JSValueRef leapForwardCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
 {
-    // FIXME: Add proper support for forward leaps
+    if (argumentCount > 0) {
+        msgQueue[endOfQueue].delay = JSValueToNumber(context, arguments[0], exception);
+        timeOffset += msgQueue[endOfQueue].delay;
+        ASSERT(!exception || !*exception);
+    }
+
     return JSValueMakeUndefined(context);
 }
 
+bool prepareMouseButtonEvent(GdkEvent* event, int button)
+{
+    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
+    if (!view)
+        return false;
+
+    memset(event, 0, sizeof(event));
+    event->button.button = button;
+    event->button.x = lastMousePositionX;
+    event->button.y = lastMousePositionY;
+    event->button.window = GTK_WIDGET(view)->window;
+    event->button.device = gdk_device_get_core_pointer();
+    event->button.state = getStateFlags();
+
+    // Mouse up & down events dispatched via g_signal_emit_by_name must offset
+    // their time value, so that WebKit can detect where sequences of mouse
+    // clicks begin and end. This should not interfere with GDK or GTK+ event
+    // processing, because the event is only seen by the widget.
+    event->button.time = GDK_CURRENT_TIME + timeOffset;
+
+    int xRoot, yRoot;
+#if GTK_CHECK_VERSION(2, 17, 3)
+    gdk_window_get_root_coords(GTK_WIDGET(view)->window, lastMousePositionX, lastMousePositionY, &xRoot, &yRoot);
+#else
+    getRootCoords(GTK_WIDGET(view), &xRoot, &yRoot);
+#endif
+    event->button.x_root = xRoot;
+    event->button.y_root = yRoot;
+
+    return true;
+}
+
 static JSValueRef contextClickCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
 {
-    webkit_web_frame_layout(mainFrame);
-
-    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
-    if (!view)
+    GdkEvent event;
+    if (!prepareMouseButtonEvent(&event, 3))
         return JSValueMakeUndefined(context);
 
-    GdkEvent event;
-    memset(&event, 0, sizeof(event));
-    event.button.button = 3;
-    event.button.x = lastMousePositionX;
-    event.button.y = lastMousePositionY;
-    event.button.window = GTK_WIDGET(view)->window;
-
-    gboolean return_val;
-    down = true;
     event.type = GDK_BUTTON_PRESS;
-    g_signal_emit_by_name(view, "button_press_event", &event, &return_val);
-
-    down = false;
+    sendOrQueueEvent(event);
     event.type = GDK_BUTTON_RELEASE;
-    g_signal_emit_by_name(view, "button_release_event", &event, &return_val);
+    sendOrQueueEvent(event);
 
     return JSValueMakeUndefined(context);
 }
 
-static void updateClickCount(int /* button */)
+static void updateClickCount(int button)
 {
-    // FIXME: take the last clicked button number and the time of last click into account.
-    if (lastClickPositionX != lastMousePositionX || lastClickPositionY != lastMousePositionY)
+    if (lastClickPositionX != lastMousePositionX
+        || lastClickPositionY != lastMousePositionY
+        || lastClickButton != button
+        || timeOffset - lastClickTimeOffset >= 1)
         clickCount = 1;
     else
         clickCount++;
@@ -146,111 +183,69 @@ static void getRootCoords(GtkWidget* view, int* rootX, int* rootY)
 
 static JSValueRef mouseDownCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
 {
-    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
-    if (!view)
-        return JSValueMakeUndefined(context);
-
-    down = true;
-
-    GdkEvent event;
-    memset(&event, 0, sizeof(event));
-    event.type = GDK_BUTTON_PRESS;
-    event.button.button = 1;
-
+    int button = 1;
     if (argumentCount == 1) {
-        event.button.button = (int)JSValueToNumber(context, arguments[0], exception) + 1;
+        button = static_cast<int>(JSValueToNumber(context, arguments[0], exception)) + 1;
         g_return_val_if_fail((!exception || !*exception), JSValueMakeUndefined(context));
     }
 
-    currentEventButton = event.button.button;
+    GdkEvent event;
+    if (!prepareMouseButtonEvent(&event, button))
+        return JSValueMakeUndefined(context);
 
-    event.button.x = lastMousePositionX;
-    event.button.y = lastMousePositionY;
-    event.button.window = GTK_WIDGET(view)->window;
-    event.button.time = GDK_CURRENT_TIME;
-    event.button.device = gdk_device_get_core_pointer();
+    buttonCurrentlyDown = event.button.button;
 
-    int x_root, y_root;
-#if GTK_CHECK_VERSION(2,17,3)
-    gdk_window_get_root_coords(GTK_WIDGET(view)->window, lastMousePositionX, lastMousePositionY, &x_root, &y_root);
-#else
-    getRootCoords(GTK_WIDGET(view), &x_root, &y_root);
-#endif
-
-    event.button.x_root = x_root;
-    event.button.y_root = y_root;
-
-    updateClickCount(1);
-
-    if (!msgQueue[endOfQueue].delay) {
-        webkit_web_frame_layout(mainFrame);
-
-        gboolean return_val;
-        g_signal_emit_by_name(view, "button_press_event", &event, &return_val);
-        if (clickCount == 2) {
-            event.type = GDK_2BUTTON_PRESS;
-            g_signal_emit_by_name(view, "button_press_event", &event, &return_val);
-        }
-    } else {
-        // replaySavedEvents should have the required logic to make leapForward delays work
-        msgQueue[endOfQueue++].event = event;
-        replaySavedEvents();
-    }
+    // Normally GDK will send both GDK_BUTTON_PRESS and GDK_2BUTTON_PRESS for
+    // the second button press during double-clicks. WebKit GTK+ selectively
+    // ignores the first GDK_BUTTON_PRESS of that pair using gdk_event_peek.
+    // Since our events aren't ever going onto the GDK event queue, WebKit won't
+    // be able to filter out the first GDK_BUTTON_PRESS, so we just don't send
+    // it here. Eventually this code should probably figure out a way to get all
+    // appropriate events onto the event queue and this work-around should be
+    // removed.
+    updateClickCount(event.button.button);
+    if (clickCount == 2)
+        event.type = GDK_2BUTTON_PRESS;
+    else if (clickCount == 3)
+        event.type = GDK_3BUTTON_PRESS;
+    else
+        event.type = GDK_BUTTON_PRESS;
 
+    sendOrQueueEvent(event);
     return JSValueMakeUndefined(context);
 }
 
+static guint getStateFlags()
+{
+    if (buttonCurrentlyDown == 1)
+        return GDK_BUTTON1_MASK;
+    if (buttonCurrentlyDown == 2)
+        return GDK_BUTTON2_MASK;
+    if (buttonCurrentlyDown == 3)
+        return GDK_BUTTON3_MASK;
+    return 0;
+}
+
 static JSValueRef mouseUpCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
 {
-
-    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
-    if (!view)
-        return JSValueMakeUndefined(context);
-
-    down = false;
-
-    GdkEvent event;
-    memset(&event, 0, sizeof(event));
-    event.type = GDK_BUTTON_RELEASE;
-    event.button.button = 1;
-
+    int button = 1;
     if (argumentCount == 1) {
-        event.button.button = (int)JSValueToNumber(context, arguments[0], exception) + 1;
+        button = static_cast<int>(JSValueToNumber(context, arguments[0], exception)) + 1;
         g_return_val_if_fail((!exception || !*exception), JSValueMakeUndefined(context));
     }
 
-    currentEventButton = event.button.button;
-
-    event.button.x = lastMousePositionX;
-    event.button.y = lastMousePositionY;
-    event.button.window = GTK_WIDGET(view)->window;
-    event.button.time = GDK_CURRENT_TIME;
-    event.button.device = gdk_device_get_core_pointer();
-
-    int x_root, y_root;
-#if GTK_CHECK_VERSION(2,17,3)
-    gdk_window_get_root_coords(GTK_WIDGET(view)->window, lastMousePositionX, lastMousePositionY, &x_root, &y_root);
-#else
-    getRootCoords(GTK_WIDGET(view), &x_root, &y_root);
-#endif
-
-    event.button.x_root = x_root;
-    event.button.y_root = y_root;
-
-    if ((dragMode && !replayingSavedEvents) || msgQueue[endOfQueue].delay) {
-        msgQueue[endOfQueue].event = event;
-        msgQueue[endOfQueue++].isDragEvent = true;
-        replaySavedEvents();
-    } else {
-        webkit_web_frame_layout(mainFrame);
-
-        gboolean return_val;
-        g_signal_emit_by_name(view, "button_release_event", &event, &return_val);
-    }
+    GdkEvent event;
+    if (!prepareMouseButtonEvent(&event, button))
+        return JSValueMakeUndefined(context);
 
     lastClickPositionX = lastMousePositionX;
     lastClickPositionY = lastMousePositionY;
+    lastClickButton = buttonCurrentlyDown;
+    lastClickTimeOffset = timeOffset;
+    buttonCurrentlyDown = 0;
 
+    event.type = GDK_BUTTON_RELEASE;
+    sendOrQueueEvent(event);
     return JSValueMakeUndefined(context);
 }
 
@@ -273,40 +268,22 @@ static JSValueRef mouseMoveToCallback(JSContextRef context, JSObjectRef function
     event.type = GDK_MOTION_NOTIFY;
     event.motion.x = lastMousePositionX;
     event.motion.y = lastMousePositionY;
+
     event.motion.time = GDK_CURRENT_TIME;
     event.motion.window = GTK_WIDGET(view)->window;
     event.motion.device = gdk_device_get_core_pointer();
+    event.motion.state = getStateFlags();
 
-    int x_root, y_root;
+    int xRoot, yRoot;
 #if GTK_CHECK_VERSION(2,17,3)
-    gdk_window_get_root_coords(GTK_WIDGET(view)->window, lastMousePositionX, lastMousePositionY, &x_root, &y_root);
+    gdk_window_get_root_coords(GTK_WIDGET(view)->window, lastMousePositionX, lastMousePositionY, &xRoot, &yRoot);
 #else
-    getRootCoords(GTK_WIDGET(view), &x_root, &y_root);
+    getRootCoords(GTK_WIDGET(view), &xRoot, &yRoot);
 #endif
+    event.motion.x_root = xRoot;
+    event.motion.y_root = yRoot;
 
-    event.motion.x_root = x_root;
-    event.motion.y_root = y_root;
-
-    if (down) {
-        if (currentEventButton == 1)
-            event.motion.state = GDK_BUTTON1_MASK;
-        else if (currentEventButton == 2)
-            event.motion.state = GDK_BUTTON2_MASK;
-        else if (currentEventButton == 3)
-            event.motion.state = GDK_BUTTON3_MASK;
-    } else
-        event.motion.state = 0;
-
-    if (dragMode && down && !replayingSavedEvents) {
-        msgQueue[endOfQueue].event = event;
-        msgQueue[endOfQueue++].isDragEvent = true;
-    } else {
-        webkit_web_frame_layout(mainFrame);
-
-        gboolean return_val;
-        g_signal_emit_by_name(view, "motion_notify_event", &event, &return_val);
-    }
-
+    sendOrQueueEvent(event);
     return JSValueMakeUndefined(context);
 }
 
@@ -345,14 +322,7 @@ static JSValueRef mouseWheelToCallback(JSContextRef context, JSObjectRef functio
     else
         g_assert_not_reached();
 
-    if (dragMode && down && !replayingSavedEvents) {
-        msgQueue[endOfQueue].event = event;
-        msgQueue[endOfQueue++].isDragEvent = true;
-    } else {
-        webkit_web_frame_layout(mainFrame);
-        gtk_main_do_event(&event);
-    }
-
+    sendOrQueueEvent(event);
     return JSValueMakeUndefined(context);
 }
 
@@ -365,49 +335,54 @@ static JSValueRef beginDragWithFilesCallback(JSContextRef context, JSObjectRef f
     return JSValueMakeUndefined(context);
 }
 
-void replaySavedEvents()
+static void sendOrQueueEvent(GdkEvent event)
 {
-    // FIXME: This doesn't deal with forward leaps, but it should.
+    // Mouse move events are queued if the previous event was queued or if a
+    // delay was set up by leapForward().
+    if (endOfQueue != startOfQueue || msgQueue[endOfQueue].delay) {
+        msgQueue[endOfQueue++].event = event;
+        replaySavedEvents();
+        return;
+    }
 
+    dispatchEvent(event);
+}
+
+static void dispatchEvent(GdkEvent event)
+{
+    webkit_web_frame_layout(mainFrame);
     WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
     if (!view)
         return;
 
-    replayingSavedEvents = true;
+    gboolean returnValue;
+    if (event.type == GDK_BUTTON_PRESS || event.type == GDK_2BUTTON_PRESS || event.type == GDK_3BUTTON_PRESS)
+        g_signal_emit_by_name(view, "button_press_event", &event, &returnValue);
+    else if (event.type == GDK_BUTTON_RELEASE)
+        g_signal_emit_by_name(view, "button_release_event", &event, &returnValue);
+    else if (event.type == GDK_MOTION_NOTIFY)
+        g_signal_emit_by_name(view, "motion_notify_event", &event, &returnValue);
+    else if (event.type == GDK_SCROLL)
+        gtk_main_do_event(&event);
+}
 
-    for (unsigned queuePos = 0; queuePos < endOfQueue; queuePos++) {
-        GdkEvent event = msgQueue[queuePos].event;
-        gboolean return_val;
+void replaySavedEvents()
+{
+    // FIXME: Eventually we may need to have more sophisticated logic to
+    // track drag-and-drop operations.
 
-        switch (event.type) {
-        case GDK_BUTTON_RELEASE:
-            g_signal_emit_by_name(view, "button_release_event", &event, &return_val);
-            break;
-        case GDK_BUTTON_PRESS:
-            g_signal_emit_by_name(view, "button_press_event", &event, &return_val);
-            break;
-        case GDK_MOTION_NOTIFY:
-            g_signal_emit_by_name(view, "motion_notify_event", &event, &return_val);
-            break;
-        default:
-            continue;
+    // First send all the events that are ready to be sent
+    while (startOfQueue < endOfQueue) {
+        if (msgQueue[startOfQueue].delay) {
+            g_usleep(msgQueue[startOfQueue].delay * 1000);
+            msgQueue[startOfQueue].delay = 0;
         }
 
-        startOfQueue++;
-    }
-
-    int numQueuedMessages = endOfQueue - startOfQueue;
-    if (!numQueuedMessages) {
-        startOfQueue = 0;
-        endOfQueue = 0;
-        replayingSavedEvents = false;
-        return;
+        dispatchEvent(msgQueue[startOfQueue++].event);
     }
 
     startOfQueue = 0;
     endOfQueue = 0;
-
-    replayingSavedEvents = false;
 }
 
 static JSValueRef keyDownCallback(JSContextRef context, JSObjectRef function, JSObjectRef thisObject, size_t argumentCount, const JSValueRef arguments[], JSValueRef* exception)
@@ -439,66 +414,93 @@ static JSValueRef keyDownCallback(JSContextRef context, JSObjectRef function, JS
         }
     }
 
+    // handle location argument.
+    int location = DOM_KEY_LOCATION_STANDARD;
+    if (argumentCount > 2)
+        location = (int)JSValueToNumber(context, arguments[2], exception);
+
     JSStringRef character = JSValueToStringCopy(context, arguments[0], exception);
     g_return_val_if_fail((!exception || !*exception), JSValueMakeUndefined(context));
-    int gdkKeySym;
-    if (JSStringIsEqualToUTF8CString(character, "leftArrow"))
-        gdkKeySym = GDK_Left;
-    else if (JSStringIsEqualToUTF8CString(character, "rightArrow"))
-        gdkKeySym = GDK_Right;
-    else if (JSStringIsEqualToUTF8CString(character, "upArrow"))
-        gdkKeySym = GDK_Up;
-    else if (JSStringIsEqualToUTF8CString(character, "downArrow"))
-        gdkKeySym = GDK_Down;
-    else if (JSStringIsEqualToUTF8CString(character, "pageUp"))
-        gdkKeySym = GDK_Page_Up;
-    else if (JSStringIsEqualToUTF8CString(character, "pageDown"))
-        gdkKeySym = GDK_Page_Down;
-    else if (JSStringIsEqualToUTF8CString(character, "home"))
-        gdkKeySym = GDK_Home;
-    else if (JSStringIsEqualToUTF8CString(character, "end"))
-        gdkKeySym = GDK_End;
-    else if (JSStringIsEqualToUTF8CString(character, "delete"))
-        gdkKeySym = GDK_BackSpace;
-    else if (JSStringIsEqualToUTF8CString(character, "F1"))
-        gdkKeySym = GDK_F1;
-    else if (JSStringIsEqualToUTF8CString(character, "F2"))
-        gdkKeySym = GDK_F2;
-    else if (JSStringIsEqualToUTF8CString(character, "F3"))
-        gdkKeySym = GDK_F3;
-    else if (JSStringIsEqualToUTF8CString(character, "F4"))
-        gdkKeySym = GDK_F4;
-    else if (JSStringIsEqualToUTF8CString(character, "F5"))
-        gdkKeySym = GDK_F5;
-    else if (JSStringIsEqualToUTF8CString(character, "F6"))
-        gdkKeySym = GDK_F6;
-    else if (JSStringIsEqualToUTF8CString(character, "F7"))
-        gdkKeySym = GDK_F7;
-    else if (JSStringIsEqualToUTF8CString(character, "F8"))
-        gdkKeySym = GDK_F8;
-    else if (JSStringIsEqualToUTF8CString(character, "F9"))
-        gdkKeySym = GDK_F9;
-    else if (JSStringIsEqualToUTF8CString(character, "F10"))
-        gdkKeySym = GDK_F10;
-    else if (JSStringIsEqualToUTF8CString(character, "F11"))
-        gdkKeySym = GDK_F11;
-    else if (JSStringIsEqualToUTF8CString(character, "F12"))
-        gdkKeySym = GDK_F12;
-    else {
-        int charCode = JSStringGetCharactersPtr(character)[0];
-        if (charCode == '\n' || charCode == '\r')
-            gdkKeySym = GDK_Return;
-        else if (charCode == '\t')
-            gdkKeySym = GDK_Tab;
-        else if (charCode == '\x8')
+    int gdkKeySym = GDK_VoidSymbol;
+    if (location == DOM_KEY_LOCATION_NUMPAD) {
+        if (JSStringIsEqualToUTF8CString(character, "leftArrow"))
+            gdkKeySym = GDK_KP_Left;
+        else if (JSStringIsEqualToUTF8CString(character, "rightArrow"))
+            gdkKeySym = GDK_KP_Right;
+        else if (JSStringIsEqualToUTF8CString(character, "upArrow"))
+            gdkKeySym = GDK_KP_Up;
+        else if (JSStringIsEqualToUTF8CString(character, "downArrow"))
+            gdkKeySym = GDK_KP_Down;
+        else if (JSStringIsEqualToUTF8CString(character, "pageUp"))
+            gdkKeySym = GDK_KP_Page_Up;
+        else if (JSStringIsEqualToUTF8CString(character, "pageDown"))
+            gdkKeySym = GDK_KP_Page_Down;
+        else if (JSStringIsEqualToUTF8CString(character, "home"))
+            gdkKeySym = GDK_KP_Home;
+        else if (JSStringIsEqualToUTF8CString(character, "end"))
+            gdkKeySym = GDK_KP_End;
+        else
+            // Assume we only get arrow/pgUp/pgDn/home/end keys with
+            // location=NUMPAD for now.
+            g_assert_not_reached();
+    } else {
+        if (JSStringIsEqualToUTF8CString(character, "leftArrow"))
+            gdkKeySym = GDK_Left;
+        else if (JSStringIsEqualToUTF8CString(character, "rightArrow"))
+            gdkKeySym = GDK_Right;
+        else if (JSStringIsEqualToUTF8CString(character, "upArrow"))
+            gdkKeySym = GDK_Up;
+        else if (JSStringIsEqualToUTF8CString(character, "downArrow"))
+            gdkKeySym = GDK_Down;
+        else if (JSStringIsEqualToUTF8CString(character, "pageUp"))
+            gdkKeySym = GDK_Page_Up;
+        else if (JSStringIsEqualToUTF8CString(character, "pageDown"))
+            gdkKeySym = GDK_Page_Down;
+        else if (JSStringIsEqualToUTF8CString(character, "home"))
+            gdkKeySym = GDK_Home;
+        else if (JSStringIsEqualToUTF8CString(character, "end"))
+            gdkKeySym = GDK_End;
+        else if (JSStringIsEqualToUTF8CString(character, "delete"))
             gdkKeySym = GDK_BackSpace;
+        else if (JSStringIsEqualToUTF8CString(character, "F1"))
+            gdkKeySym = GDK_F1;
+        else if (JSStringIsEqualToUTF8CString(character, "F2"))
+            gdkKeySym = GDK_F2;
+        else if (JSStringIsEqualToUTF8CString(character, "F3"))
+            gdkKeySym = GDK_F3;
+        else if (JSStringIsEqualToUTF8CString(character, "F4"))
+            gdkKeySym = GDK_F4;
+        else if (JSStringIsEqualToUTF8CString(character, "F5"))
+            gdkKeySym = GDK_F5;
+        else if (JSStringIsEqualToUTF8CString(character, "F6"))
+            gdkKeySym = GDK_F6;
+        else if (JSStringIsEqualToUTF8CString(character, "F7"))
+            gdkKeySym = GDK_F7;
+        else if (JSStringIsEqualToUTF8CString(character, "F8"))
+            gdkKeySym = GDK_F8;
+        else if (JSStringIsEqualToUTF8CString(character, "F9"))
+            gdkKeySym = GDK_F9;
+        else if (JSStringIsEqualToUTF8CString(character, "F10"))
+            gdkKeySym = GDK_F10;
+        else if (JSStringIsEqualToUTF8CString(character, "F11"))
+            gdkKeySym = GDK_F11;
+        else if (JSStringIsEqualToUTF8CString(character, "F12"))
+            gdkKeySym = GDK_F12;
         else {
-            gdkKeySym = gdk_unicode_to_keyval(charCode);
-            if (WTF::isASCIIUpper(charCode))
-                state |= GDK_SHIFT_MASK;
+            int charCode = JSStringGetCharactersPtr(character)[0];
+            if (charCode == '\n' || charCode == '\r')
+                gdkKeySym = GDK_Return;
+            else if (charCode == '\t')
+                gdkKeySym = GDK_Tab;
+            else if (charCode == '\x8')
+                gdkKeySym = GDK_BackSpace;
+            else {
+                gdkKeySym = gdk_unicode_to_keyval(charCode);
+                if (WTF::isASCIIUpper(charCode))
+                    state |= GDK_SHIFT_MASK;
+            }
         }
     }
-
     JSStringRelease(character);
 
     WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
@@ -512,8 +514,18 @@ static JSValueRef keyDownCallback(JSContextRef context, JSObjectRef function, JS
     event.key.state = state;
     event.key.window = GTK_WIDGET(view)->window;
 
+    // When synthesizing an event, an invalid hardware_keycode value
+    // can cause it to be badly processed by Gtk+.
+    GdkKeymapKey* keys;
+    gint n_keys;
+    if (gdk_keymap_get_entries_for_keyval(gdk_keymap_get_default(), gdkKeySym, &keys, &n_keys)) {
+        event.key.hardware_keycode = keys[0].keycode;
+        g_free(keys);
+    }
+
     gboolean return_val;
     event.key.type = GDK_KEY_PRESS;
+
     g_signal_emit_by_name(view, "key-press-event", &event.key, &return_val);
 
     event.key.type = GDK_KEY_RELEASE;
@@ -604,17 +616,24 @@ static JSClassRef getClass(JSContextRef context)
     return eventSenderClass;
 }
 
-JSObjectRef makeEventSender(JSContextRef context)
+JSObjectRef makeEventSender(JSContextRef context, bool isTopFrame)
 {
-    down = false;
-    dragMode = true;
-    lastMousePositionX = lastMousePositionY = 0;
-    lastClickPositionX = lastClickPositionY = 0;
+    if (isTopFrame) {
+        dragMode = true;
+
+        // Fly forward in time one second when the main frame loads. This will
+        // ensure that when a test begins clicking in the same location as
+        // a previous test, those clicks won't be interpreted as continuations
+        // of the previous test's click sequences.
+        timeOffset += 1000;
+
+        lastMousePositionX = lastMousePositionY = 0;
+        lastClickPositionX = lastClickPositionY = 0;
+        lastClickTimeOffset = 0;
+        lastClickButton = 0;
+        buttonCurrentlyDown = 0;
+        clickCount = 0;
 
-    if (!replayingSavedEvents) {
-        // This function can be called in the middle of a test, even
-        // while replaying saved events. Resetting these while doing that
-        // can break things.
         endOfQueue = 0;
         startOfQueue = 0;
     }
